/*
 * <security/pam_modules.h>
 * 
 * This header file documents the PAM SPI --- that is, interface
 * between the PAM library and a PAM service library which is called
 * by the PAM library.
 *
 * Note, the copyright information is at end of file.
 *
 * Created: 15-Jan-96 by TYT
 * Last modified: 1996/3/5 by AGM
 *
 * $Log: pam_modules.h,v $
 * Revision 1.3  1996/03/16 20:32:40  morgan
 * changed < ... > title
 * added const to last argument of pam_get_data
 * moved some tokens from ./_pam_types.h
 *
 * Revision 1.2  1996/03/09 20:36:02  morgan
 * added RCS information
 *
 *
 * $Id: pam_modules.h,v 1.3 1996/03/16 20:32:40 morgan Exp $
 *
 */

#ifndef _SECURITY_PAM_MODULES_H
#define _SECURITY_PAM_MODULES_H

#include <security/_pam_types.h>      /* Linux-PAM common defined types */

/* these defines are used by pam_set_item() and pam_get_item() and are
 * in addition to those found in <security/_pam_types.h> */

#define PAM_AUTHTOK     6	/* The authentication token (password) */
#define PAM_OLDAUTHTOK  7	/* The old authentication token */

/* -------------- The Linux-PAM Module PI ------------- */

extern int pam_set_data(pam_handle_t *pamh, const char *module_data_name,
			void *data,
			int (*cleanup)(pam_handle_t *pamh, void *data,
				       int error_status));
extern int pam_get_data(const pam_handle_t *pamh,
			const char *module_data_name, const void **data);

#ifdef PAM_STATIC_LIBRARY
#define PAM_EXTERN static

struct pam_module {
    char *name;		/* Name of the module */

    /* These are function pointers to the module's key functions.  */

    int (*pam_sm_authenticate)(pam_handle_t *pamh, int flags,
			       int argc, const char **argv);
    int (*pam_sm_setcred)(pam_handle_t *pamh, int flags,
			  int argc, const char **argv);
    int (*pam_sm_acct_mgmt)(pam_handle_t *pamh, int flags,
			    int argc, const char **argv);
    int (*pam_sm_open_session)(pam_handle_t *pamh, int flags,
			       int argc, const char **argv);
    int (*pam_sm_close_session)(pam_handle_t *pamh, int flags,
				int argc, const char **argv);
    int (*pam_sm_chauthtok)(pam_handle_t *pamh, int flags,
			    int argc, const char **argv);

    struct pam_modules *prev, *next;         /* linked-list pointers */
}

#else /* !PAM_STATIC_LIBRARY */

#define PAM_EXTERN extern

#endif /* PAM_STATIC LIBRARY */
	
/* Authentication API's */
PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh, int flags,
				   int argc, const char **argv);
PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags,
			      int argc, const char **argv);

/* Account Management API's */
PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
				int argc, const char **argv);

/* Session Management API's */
PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags,
				   int argc, const char **argv);

PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags,
				    int argc, const char **argv);

/* Password Management API's */
PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
				int argc, const char **argv);


/* The following two flags are for use across the Linux-PAM/module
 * interface only. The Application is not permitted to use these
 * tokens.
 *
 * The password service should only perform preliminary checks.  No
 * passwords should be updated. */
#define PAM_PRELIM_CHECK		0x4000

/* The password service should update passwords Note: PAM_PRELIM_CHECK
 * and PAM_UPDATE_AUTHTOK can not both be set simultaneously! */
#define PAM_UPDATE_AUTHTOK		0x2000

#undef PAM_EXTERN

/* Copyright Theodore Ts'o, 1996.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, and the entire permission notice in its entirety,
 *    including the disclaimer of warranties.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 * 
 * ALTERNATIVELY, this product may be distributed under the terms of
 * the GNU Public License, in which case the provisions of the GPL are
 * required INSTEAD OF the above restrictions.  (This clause is
 * necessary due to a potential bad interaction between the GPL and
 * the restrictions contained in a BSD-style copyright.)
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.  */

#endif /* _SECURITY_PAM_MODULES_H */

